
BUFFR   = 0x10		# Stack offset for buffer
LENGTH  = 0x14		# Stack offset for length
RTC_INX = 0x70		# Realtime clock index register
RTC_DAT = 0x71		# Realtime clock data port
CMD_STA = 0x0b		# Command and status register
DEV_HLT = 0x80		# Device halt bit
INT_ENA = 0x40		# Periodic interrupt

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
.section	.data
.global	rtc_tick
.type	rtc_tick,@object
rtc_tick:
rtc_tickl:	.long   0x00000000	
rtc_tickh:	.long	0x00000000
.size	rtc_tickl,4
.size	rtc_tickh,4
.section	.text
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
# 	This resets the RTC chip for the next interrupt, while leaving
#	the index register at 0 so power-down only trashes seconds,
#	not something that is check-summed.
#
.macro	reset_rtc
	movb	$0x0b, %al		# Control register B
	outb	%al, $RTC_INX		# Index port
	call	delay
	inb	$RTC_DAT, %al		# Get current
	call	delay
	testb	$DEV_HLT, %al		# See if halted
	jz	0f			# It's running
	andb	$~DEV_HLT, %al		# Stopped, reset the bit
	outb	%al, $RTC_DAT		# Restart it
	call	delay
0:	movb	$0x0c, %al		# Status register C
	outb	%al, $RTC_INX		# Index port
	call	delay
	inb	$RTC_DAT, %al		# This ACKs the interrupt
	call	delay
	movb	$0x00, %al		# Seconds index 
	outb	%al, $RTC_INX		# Index port
	call	delay
.endm
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#	This produces a calibrated delay between setting the index
#	register of the RTC chip and reading/writing to the data
#	port.
#
delay:	pushl	%eax		# Save register
	inb	$0x80, %al	# Read port byte
	popl	%eax
	ret

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#	This clears the timer-tick of the RTC clock. It doesn't disable
#	the timer so it does low-word first to make sure it doesn't
#	wrap when being cleared.
#
.global	clear_rtc
.type	clear_rtc,@function
.align	0x04
clear_rtc:
	xorl	%eax, %eax
1:	movl	%eax,(rtc_tickl)
	movl	%eax,(rtc_tickh)
	cmpl	%eax,(rtc_tickl)
	jnz	1b
	cmpl	%eax,(rtc_tickh)
	jnz	1b
	ret

#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  This starts the RTC interrupting at the programmed timer-rate.
#
.global init_rtc
.type init_rtc,@function
.align	0x04
init_rtc:
	movb	$0x0a, %al		# Control register 'A'
	outb	%al, $RTC_INX		# Index port
	call	delay
	movb	$0x25, %al
	outb	%al, $RTC_DAT		# Data port
	call	delay
	movb	$0x0b, %al		# Control register 'B'
	outb	%al, $RTC_INX		# Index port
	call	delay
	inb	$RTC_DAT, %al		# Get current
	call	delay
	andb	$~DEV_HLT, %al
	orb	$INT_ENA, %al		# Enable periodic interrupt
	outb	%al, $RTC_DAT		# Write to data port
	call	delay
	reset_rtc
	ret
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#  This stops the RTC from interrupting at the programmed timer-rate.
#
.global stop_rtc
.type stop_rtc,@function
.align	0x04
stop_rtc:
	movb	$0x0b, %al		# Status register
	outb	%al, $RTC_INX		# Index port
	call	delay	
	inb	$RTC_DAT, %al		# Get current
	call	delay
	andb	$~(INT_ENA|DEV_HLT),%al	# Disable periodic interrupt
	outb	%al, $RTC_DAT		# Write to data port
	call	delay
	movb	$0x00, %al		# Seconds index 
	outb	%al, $RTC_INX		# Index port
	call	delay
	ret
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#   This is the actual ISR code for the RTC.
#
.global rtc_isr
.type rtc_isr,@function
.align	0x04
rtc_isr:
	addl	$1,(rtc_tickl)
	adcl	$0,(rtc_tickh)
	reset_rtc
	ret
.end
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
